/* Terraform - (C) 1997-1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "HeightField.h" 
#include "GlobalTrace.h"
#include "GlobalSanityCheck.h"



int	HeightField::s_HFcount = 0;


/*
 *  constructor: initialize everying to a meaningful default value
 */
HeightField::HeightField (PTYPE *hf, int xsize, int ysize, char *name) 
	    : HeightFieldCore (hf, xsize, ysize)
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ HeightField\n");

	if (hf)
		{
		d_updateTime = time(NULL);
		SanityCheck::bailout ((!xsize), "xsize==0", "HeightField::HeightField");
		SanityCheck::bailout ((!ysize), "ysize==0", "HeightField::HeightField");
		}
	else
		{
		SanityCheck::bailout ((xsize), "xsize!=0", "HeightField::HeightField");
		SanityCheck::bailout ((ysize), "ysize!=0", "HeightField::HeightField");
		d_updateTime = 0;
		}

	if (name)
		d_name = strdup (name);
	else
		d_name = getNewName (); 

	d_hfBck=NULL;
	d_contourList=NULL;
	d_loadedFromFile=FALSE;
	d_isSaved=TRUE;
	d_backupTime = 0;
}


/*
 *  destructor: clean up, free allocated memory
 */
HeightField::~HeightField ()
{
	if (d_name)
		free (d_name);
	destroyBackup ();

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- HeightField\n");
}


/*
 *  setName: assign a new (non-NULL) Name 
 */
void HeightField::setName (char *name)
{
	SanityCheck::bailout ((!name), "name==NULL", "HeightField::setName");

	if (d_name)
		free (d_name);
	d_name = strdup (name);
}



/*
 *  setSaved: set the d_isSaved flag
 */
void HeightField::setSaved (bool isSaved)
{
	d_isSaved = isSaved;
	d_updateTime = time(NULL);

	if (!isSaved && d_contourList)		// expire contour list
		setContourList (NULL, time (NULL));
}


/*
 *  setContourList: assign a new contour list and delete the old one
 */
void HeightField::setContourList(TFGListFlexArr *l, time_t timestamp)
{
	// delete old list
	if (d_contourList)
		delete d_contourList;

	d_contourList = l;
	d_clTimestamp = timestamp;
}


/*
 *  getNewName: return a newly allocated string for the next name.
 * 	receiver becomes the owner of this piece of memory!
 */
char *HeightField::getNewName ()
{
	char	buf[256];
	char	*newName;

	sprintf (buf, "%s-%02d.tga", TF_DEFAULT_FILENAME, ++s_HFcount);
	newName = strdup (buf);
	return (newName);
}


/*
 *  getCopyOfData: return a newly allocated copy of the actual data stream. 
 * 	The receiver becomes the owner of this piece of memory!
 */
PTYPE *HeightField::getCopyOfData ()
{
	SanityCheck::bailout ((!d_hf), "d_hf==NULL", "HeightField::getCopyOfData");

	PTYPE	 *hfnew = new PTYPE[d_size];

	memcpy (hfnew, d_hf, d_size*sizeof(PTYPE));
	return hfnew;
}


/*
 *  backup: make a backup copy of the current height field
 */
void HeightField::backup ()
{
	SanityCheck::bailout ((!d_hf), "d_hf==NULL", "HeightField::backup");

	if (d_hfBck)
		delete [] d_hfBck;
	d_hfBck = new PTYPE[d_size];
	memcpy (d_hfBck, this->d_hf, sizeof(PTYPE)*this->d_size);
	d_backupTime = d_updateTime;
}


/*
 *  restore: restore a backup copy of the current height field. Doesn't
 *  	     calculate statistics! 
 */
void HeightField::restore ()
{
	SanityCheck::bailout ((!d_hf), "d_hf==NULL", "HeightField::restore");
	SanityCheck::bailout ((!d_hfBck), "d_hfBck==NULL", "HeightField::restore");

	memcpy (this->d_hf, d_hfBck, sizeof(PTYPE)*this->d_size);
	d_updateTime = d_backupTime;
}


/*
 *  destroyBackup: destroy the backup copy of the current height field and 
 * 	free the memory. 
 */
void HeightField::destroyBackup ()
{
	if (!d_hfBck)
		return;

	delete [] d_hfBck;
	d_hfBck = NULL;
	d_backupTime = 0;
}


